(function() {
    "use strict";

    // 会社アプリ操作
    var zipDao = {};

    function createAddressUpdateInfoList(appId, opt_getRecords, opt_offset, opt_putRecords) {
        var offset = opt_offset || 0;
        var putRecords = opt_putRecords || [];
        // offset番目のレコードを取り出し
        var record = opt_getRecords.records[offset];
        // ハイフン有無に対応
        var postcode = record['郵便番号']['value'];
        // 前半３桁
        var post3 = '';
        // 後半４桁
        var post4 = '';
        if (postcode.length === 7) {
            post3 = postcode.slice(0, 3);
            post4 = postcode.slice(3, 7);
        } else if (postcode.length === 8) {
            post3 = postcode.slice(0, 3);
            post4 = postcode.slice(4, 8);
        }
        var url = 'https://madefor.github.io/postal-code-api/api/v1/' + post3 + '/' + post4 + '.json';

        // 外部API呼出
        return kintone.proxy(url, 'GET', {}, {}).then(function(resp) {
            // 成功時
            if (resp[1] === 200) {
                // ステータス正常時
                // JSON文字列をJSONオブジェクトに変換
                var obj = JSON.parse(resp[0]);
                // 更新パラメータの組み立て
                var params = {
                    "id": record['$id']['value'],
                    "record": {
                        "都道府県": {
                            "value": obj['data'][0]['ja']['prefecture']
                        },
                        "市区町村": {
                            "value": obj['data'][0]['ja']['address1']
                        },
                        "町域名": {
                            "value": obj['data'][0]['ja']['address2']
                        }
                    }
                };
                putRecords.push(params);
                if (offset !== opt_getRecords.records.length - 1) {
                    return createAddressUpdateInfoList(appId, opt_getRecords, offset + 1, putRecords);
                }
            } else {
                // ステータス異常時
                try {
                    // すべての更新を中止
                    putRecords = [];
                    throw new Error('レコード番号' + record['$id']['value'] + ': 住所が見つかりませんでした。');
                } catch (e) {
                    alert(e.name + ': ' + e.message);
                }
            }
            return putRecords;
        }, function(resp) {
            alert("レコードの取得に失敗しました。");
        });
    }

    // 一覧画面の表示後イベント
    kintone.events.on('app.record.index.show', function(event) {
        // すでに住所検索ボタンがあれば何もしない
        if (document.getElementById('btnAddressUpdate') !== null) {
            return event;
        }

        // 一覧画面に住所検索ボタンを表示
        // HTMLボタン要素生成
        var btnAddressUpdate = document.createElement('button');
        // ボタン識別ID
        btnAddressUpdate.id = 'btnAddressUpdate';
        // ボタン名
        btnAddressUpdate.innerHTML = '住所検索';
        // ボタンタイプ
        btnAddressUpdate.type = 'button';
        // ボタンクリックイベント
        btnAddressUpdate.onclick = function() {
            if (window.confirm('住所検索を実行しますか？')) {
                // 表示中の一覧画面の絞り込み条件で住所検索の対象レコードを取得
                zipDao.select().then(function(zipRecords) {
                    // 成功時
                    // 100レコードを超えた場合は戻る
                    if (zipRecords.length > 100) {
                        alert("レコード件数が100件を超えているため中止します。");
                    } else {
                        // 再帰呼び出しで住所検索
                        createAddressUpdateInfoList(kintone.app.getId(), zipRecords).then(function(addressList) {
                            if (addressList.length !== 0) {
                                // 成功時
                                zipDao.update(addressList);
                            } else {
                                // 失敗時
                                alert("住所検索を中止しました。");
                            }
                        });
                    }
                }, function(zipRecords) {
                    // 失敗時
                    alert("レコードの取得に失敗しました。");
                });
            }
        };
        // メニューの右側にボタン要素を追加
        kintone.app.getHeaderMenuSpaceElement().appendChild(btnAddressUpdate);
        return event;
    });

    // 表示中の一覧画面の絞り込み条件で住所検索の対象レコードを取得
    zipDao.select = function() {
        var queryCondition = kintone.app.getQueryCondition();
        var params = {
            "app": kintone.app.getId(),
            "query": queryCondition + ' order by レコード番号 asc'
        };

        return kintone.api('/k/v1/records', 'GET', params);
    };

    // 各レコードの都道府県、市区町村、町域名を更新
    zipDao.update = function(addressList) {
        var params = {
            "app": kintone.app.getId(),
            "records": addressList
        };
        // 住所を一括更新して画面を再表示
        kintone.api('/k/v1/records', 'PUT', params).then(function(resp) {
            // 成功時
            // alert("住所検索を完了しました。");
            // 画面リフレッシュ
            location.reload();
        }, function(resp) {
            // 失敗時
            alert("住所の更新に失敗しました。");
        });
    };
})();
